/*
          LTR24.
            ,    
           :
    ltr24_recv  slot crate_serial srvip
    :
        slot         -   ( 1  16)
        crate_serial -    (   )
        srvip        - ip-  ltrd  LtrServer (     )
     :    ,       

              .
             .

         ,   
       .

           Windows   CTRL+C  Linux

      VisualStudio:
          Visual Studio,     
    ( (Project) ->  (Properties) ->   (Configuration Properties)
    -> /++ ->  (General) ->    (Additional Include Directories))
            (ltr24api.h    
         ltr/include   )
        .lib   <  >/lib/msvc
    ( (Project) ->  (Properties) ->   (Configuration Properties) ->
     (Linker) ->  (General) ->    (Additional Library Directories)).

    !:      Visual Studio    
       ,    :
      (File) ->    (Advanced Save Options)...
        (Encoding)   (UTF8,  )/Unicode (UTF-8 with signature)
        .    ,   
           ( Consolas).
*/

#include "ltr/include/ltr24api.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef _WIN32
#include <locale.h>
#include <conio.h>
#else
#include <signal.h>
#include <unistd.h>
#endif


/*    ,    */
#define RECV_BLOCK_CH_SIZE  4096*3
/*       */
#define RECV_TOUT  4000


typedef struct {
    int slot;
    const char *serial;
    DWORD addr;
    unsigned char freq_rate;
} t_open_param;


/*      */
static int f_out = 0;

#ifndef _WIN32
/*     Linux */
static void f_abort_handler(int sig) {
    f_out = 1;
}
#endif


/*------------------------------------------------------------------------------------------------*/
static int f_get_params(int argc, char **argv, t_open_param *par) {
    /*    .   ,  
     *   :
     * 1  -   ( 1  16)
     * 2  -   
     * 3  -   
     * 4  - ip- 
     */
    int err = 0;

    par->slot = LTR_CC_CHNUM_MODULE1;
    par->freq_rate = LTR24_FREQ_117K;
    par->serial = "";
    par->addr = LTRD_ADDR_DEFAULT;


    if (argc > 1)
        par->slot = atoi(argv[1]);
    if (argc > 2)
        par->freq_rate = atoi(argv[2]);
    if (argc > 3)
        par->serial = argv[3];
    if (argc > 4) {
        int a[4];
        int i;
        if (sscanf(argv[4], "%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3]) != 4) {
            fprintf(stderr, "  IP-!!\n");
            err = -1;
        }

        for (i = 0; ((i < 4) && !err); i++) {
            if ((a[i] < 0) || (255 < a[i])) {
                fprintf(stderr, " IP-!!\n");
                err = -1;
            }
        }

        if (!err)
            par->addr = (a[0] << 24) | (a[1] << 16) | (a[2] << 8) | a[3];
    }

    return err;
}

int main(int argc, char** argv) {
    INT err = LTR_OK;
    TLTR24 hltr24;
    t_open_param par;
#ifndef _WIN32
    struct sigaction sa;
    memset(&sa, 0, sizeof(sa));
    /*   Linux      ,
           */
    sa.sa_handler = f_abort_handler;
    sigaction(SIGTERM, &sa, NULL);
    sigaction(SIGINT, &sa, NULL);
    sigaction(SIGABRT, &sa, NULL);
#endif

#ifdef _WIN32
    /*         Windows*/
    setlocale(LC_ALL, "");
#endif

    err = f_get_params(argc, argv, &par);

    if (err == LTR_OK) {
        LTR24_Init(&hltr24);
        /*     */
        err = LTR24_Open(&hltr24, par.addr, LTRD_PORT_DEFAULT, par.serial, par.slot);
        if (err != LTR_OK) {
            fprintf(stderr, "     .  %d (%s)\n",
                    err, LTR24_GetErrorString(err));
        } else {
            INT ch_cnt=0;
            INT close_err;

            /*    ,  ,  Flash-
             * .        
             *   */
            err = LTR24_GetConfig(&hltr24);
            if (err != LTR_OK) {
                fprintf(stderr, "       Flash-.  %d (%s)\n",
                        err, LTR24_GetErrorString(err));
            } else {
                /*      */
                printf("  !   : \n");
                printf("       = %s\n", hltr24.ModuleInfo.Name);
                printf("        = %s\n", hltr24.ModuleInfo.Serial);
                printf("   PLD         = %d\n", hltr24.ModuleInfo.VerPLD);
                printf("   ICP      = %s\n", hltr24.ModuleInfo.SupportICP ? "" : "");
                fflush(stdout);
            }

            /*   */
            if (err == LTR_OK) {
                /*  - 24  20  */
                hltr24.DataFmt = LTR24_FORMAT_20;
                /*        ( 24- 
                   .    2- ,  4 -   58)  */
                hltr24.ADCFreqCode = par.freq_rate;
                /* ./.   ( /ICP-) */
                hltr24.TestMode = FALSE;

                /*    */
                hltr24.ChannelMode[0].Enable = TRUE;
                hltr24.ChannelMode[0].Range = LTR24_RANGE_10;
                hltr24.ChannelMode[0].AC = FALSE;
                hltr24.ChannelMode[0].ICPMode = TRUE;

                hltr24.ChannelMode[1].Enable = TRUE;
                hltr24.ChannelMode[1].Range = LTR24_RANGE_2;
                hltr24.ChannelMode[1].AC = TRUE;
                hltr24.ChannelMode[1].ICPMode = TRUE;

                hltr24.ChannelMode[2].Enable = TRUE;
                hltr24.ChannelMode[2].Range = LTR24_RANGE_2;
                hltr24.ChannelMode[2].AC = TRUE;
                hltr24.ChannelMode[2].ICPMode = TRUE;

                hltr24.ChannelMode[3].Enable = FALSE;

                err = LTR24_SetADC(&hltr24);
                if (err != LTR_OK) {
                    fprintf(stderr, "    :  %d (%s)\n",
                            err, LTR24_GetErrorString(err));
                } else  {
                    INT i;
                    /*  -   */
                    for (i = 0, ch_cnt=0; i < LTR24_CHANNEL_NUM; i++) {
                        if (hltr24.ChannelMode[i].Enable)
                            ch_cnt++;
                    }

                    /*  SetADC()   AdcFreq.   
                     *   */
                    printf("   .  = %.2f , -  = %d\n",
                            hltr24.ADCFreq, ch_cnt);
                }
            }

            if (err == LTR_OK) {
                DWORD recvd_blocks=0;
                INT recv_data_cnt = RECV_BLOCK_CH_SIZE*ch_cnt;
                /*  24-        ,
                     20- -  */
                INT   recv_wrd_cnt = recv_data_cnt*(hltr24.DataFmt==LTR24_FORMAT_24 ? 2 : 1);
                DWORD  *rbuf = (DWORD*)malloc(recv_wrd_cnt*sizeof(rbuf[0]));
                double *data = (double *)malloc(recv_data_cnt*sizeof(data[0]));
                BOOL   *ovlds = (BOOL *)malloc(recv_data_cnt*sizeof(ovlds[0]));

                if ((rbuf == NULL) || (data == NULL) || (ovlds == NULL)) {
                    fprintf(stderr, "  !\n");
                    err = LTR_ERROR_MEMORY_ALLOC;
                }

                if (err == LTR_OK) {
                    /*    */
                    err = LTR24_Start(&hltr24);
                    if (err != LTR_OK) {
                        fprintf(stderr, "    !  %d (%s)\n",
                                err, LTR24_GetErrorString(err));
                    }
                }

                if (err == LTR_OK) {
                    printf("  .    %s\n",
#ifdef _WIN32
                           " "
#else
                           "CTRL+C"
#endif
                           );
                    fflush(stdout);
                }


                /*        
                 *    */
                while (!f_out && (err == LTR_OK)) {
                    INT recvd;
                    /*         */
                    DWORD tout = RECV_TOUT + (DWORD)(1000.*RECV_BLOCK_CH_SIZE/hltr24.ADCFreq + 1);

                    /*    .  */
                    recvd = LTR24_Recv(&hltr24, rbuf, NULL, recv_wrd_cnt, tout);

                    /*       */
                    if (recvd < 0) {
                        err = recvd;
                        fprintf(stderr, "  .  %d:%s\n",
                                err, LTR24_GetErrorString(err));
                    } else if (recvd != recv_wrd_cnt) {
                        fprintf(stderr, "  .  %d,  %d\n",
                                recv_wrd_cnt*ch_cnt, recvd);
                        err = LTR_ERROR_RECV_INSUFFICIENT_DATA;
                    } else {
                        err = LTR24_ProcessData(&hltr24, rbuf, data, &recvd,
                                                LTR24_PROC_FLAG_VOLT |
                                                LTR24_PROC_FLAG_CALIBR |
                                                LTR24_PROC_FLAG_AFC_COR_EX |
                                                LTR24_PROC_FLAG_ICP_PHASE_COR,
                                                ovlds);
                        if (err != LTR_OK) {
                            fprintf(stderr, "  .  %d:%s\n",
                                    err, LTR24_GetErrorString(err));
                        } else {
                            unsigned cur_pos,ch;
                            recvd_blocks++;
                            /*       */
                            printf(" %4d:", recvd_blocks);
                            for (ch = 0, cur_pos = 0; ch < LTR24_CHANNEL_NUM; ch++) {
                                if (hltr24.ChannelMode[ch].Enable) {
                                    printf("  %d = %9.6f", ch+1, data[cur_pos]);
                                    cur_pos++;
                                }
                            }
                            printf("\n");

                            fflush(stdout);
                        }
                    }

#ifdef _WIN32
                    /*      */
                    if (err==LTR_OK) {
                        if (_kbhit())
                            f_out = 1;
                    }
#endif
                } //while (!f_out && (err==LTR_OK))

                /*    ,    */
                if (hltr24.Run) {
                    INT stop_err = LTR24_Stop(&hltr24);
                    if (stop_err != LTR_OK) {
                        fprintf(stderr, "    .  %d:%s\n",
                                stop_err, LTR24_GetErrorString(stop_err));
                        if (err == LTR_OK)
                            err = stop_err;
                    } else {
                        printf("  .\n");
                    }
                }
            }

            /*     */
            close_err = LTR24_Close(&hltr24);
            if (close_err != LTR_OK) {
                fprintf(stderr, "     .  %d:%s\n",
                        close_err, LTR24_GetErrorString(close_err));
                if (err == LTR_OK)
                    err = close_err;
            } else {
                printf("    .\n");
            }
        }
    }

    return err;
}
